package com.pravera.activity_rec_flutter.service

import DatabaseHelper
import android.annotation.SuppressLint
import android.content.Context
import android.content.Intent
import androidx.core.app.JobIntentService
import com.google.android.gms.location.ActivityRecognitionResult
import com.google.gson.Gson
import com.pravera.activity_rec_flutter.Constants
import com.pravera.activity_rec_flutter.errors.ErrorCodes
import com.pravera.activity_rec_flutter.models.ActivityData
import com.pravera.activity_rec_flutter.utils.ActivityRecognitionUtils
import com.pravera.activity_rec_flutter.utils.SharedPreferenceManager
import android.util.Log

class ActivityRecognitionIntentService : JobIntentService() {

    lateinit var mPrefs: SharedPreferenceManager
    val prefRecordedActivity = "prefRecordedActivity"
    val prefRecordedActivityTime = "prefRecordedActivityTime"
    val prefCurrentInVehiclePoints = "_prefCurrentInVehiclePoints"
    val prefCurrentOnByciclePoints = "_prefCurrentOnByciclePoints"
    val prefCurrentRunningPoints = "_prefCurrentRunningPoints"
    val prefCurrentWalkingPoints = "_prefCurrentWalkingPoints"
    val prefCurrentStillPoints = "_prefCurrentStillPoints"

    companion object {
        val jsonConverter: Gson = Gson()
        fun enqueueWork(context: Context, intent: Intent) {
            enqueueWork(
                context, ActivityRecognitionIntentService::class.java,
                Constants.ACTIVITY_RECOGNITION_INTENT_SERVICE_JOB_ID, intent
            )
        }
    }

    @SuppressLint("LongLogTag")
    override fun onHandleWork(intent: Intent) {
        val extractedResult = ActivityRecognitionResult.extractResult(intent)
        val probableActivities = extractedResult?.probableActivities
        val mostProbableActivity = probableActivities?.maxByOrNull { it.confidence } ?: return

        mPrefs = SharedPreferenceManager(this)
        val dbHelper = DatabaseHelper(this)

        val activityData = ActivityData(
            ActivityRecognitionUtils.getActivityTypeFromValue(mostProbableActivity.type),
            ActivityRecognitionUtils.getActivityConfidenceFromValue(mostProbableActivity.confidence)
        )

        if (activityData.type != "UNKNOWN") {
            var recordedActivity = mPrefs.getString(prefRecordedActivity, "")
            var recordedActivityTime = mPrefs.getLong(prefRecordedActivityTime, 0L)
            Log.d("ActivityRecognitionIntentService", "activity fetched ${activityData.type}")
            Log.d("ActivityRecognitionIntentService", "recordedActivity =====> ${recordedActivity}")
            Log.d(
                "ActivityRecognitionIntentService",
                "recordedActivityTime =====> ${recordedActivityTime}"
            )

            if (recordedActivity == "") {
                Log.d("ActivityRecognitionIntentService", " =====> recordedActivity == null")
                mPrefs.putString(prefRecordedActivity, activityData.type)
                mPrefs.putLong(prefRecordedActivityTime, System.currentTimeMillis())
            } else {
                Log.d("ActivityRecognitionIntentService", " =====> recordedActivity != null")
                if (activityData.type != recordedActivity) {
                    Log.d(
                        "ActivityRecognitionIntentService",
                        " =====> activityData != recordedActivity"
                    )
                    val timeDifferenceMillis = System.currentTimeMillis() - recordedActivityTime

                    // Convert milliseconds to minutes
                    val timeDifferenceMinutes = timeDifferenceMillis / 60000
                    mPrefs.putString(prefRecordedActivity, activityData.type)
                    mPrefs.putLong(prefRecordedActivityTime, System.currentTimeMillis())

                    when (activityData.type) {
                        "IN_VEHICLE" -> {
                            val points =
                                timeDifferenceMinutes.toInt() * 0.29 // per minute points for vehicle
//                           val storedPoints = mPrefs.getLong(prefCurrentInVehiclePoints, 0L)
//                           val currentPoints = storedPoints ?: 0L
//                           mPrefs.putLong(prefCurrentInVehiclePoints, (currentPoints + points).toLong())
                            Log.d("ActivityRecognitionIntentService", "IN_VEHICLE points -> $points")
                            val records = dbHelper.getAllActivityPoints()
                            Log.d("ActivityRecognitionIntentService", "IN_VEHICLE records -> ${records.size}")
                            Log.d("ActivityRecognitionIntentService", "IN_VEHICLE first record -> ${records.first()}")
                            if (records.isEmpty()) {
                                dbHelper.addActivityPoints(points.toInt(), 0, 0, 0, 0)
                            } else {
                                dbHelper.editActivityPoints(records.first().id,
                                    records.first().vehiclePoints + points.toInt(),
                                    records.first().bicyclePoints,
                                    records.first().runningPoints,
                                    records.first().walkingPoints,
                                    records.first().stillPoints,
                                )
                            }
                        }

                        "ON_BICYCLE" -> {
                            val points =
                                timeDifferenceMinutes.toInt() * 0.50 // per minute points for bycicle
//                            val storedPoints = mPrefs.getLong(prefCurrentOnByciclePoints, 0L)
//                            val currentPoints = storedPoints ?: 0L
//                            mPrefs.putLong(prefCurrentOnByciclePoints, (currentPoints + points).toLong())

                            Log.d("ActivityRecognitionIntentService", "ON_BICYCLE points -> $points")
                            val records = dbHelper.getAllActivityPoints()
                            Log.d("ActivityRecognitionIntentService", "ON_BICYCLE records -> ${records.size}")
                            Log.d("ActivityRecognitionIntentService", "ON_BICYCLE first record -> ${records.first()}")
                            if (records.isEmpty()) {
                                dbHelper.addActivityPoints(0, points.toInt(), 0, 0, 0)
                            } else {
                                dbHelper.editActivityPoints(records.first().id,
                                    records.first().vehiclePoints ,
                                    records.first().bicyclePoints+ points.toInt(),
                                    records.first().runningPoints,
                                    records.first().walkingPoints,
                                    records.first().stillPoints,
                                )
                            }
                        }

                        "RUNNING" -> {
                            val points =
                                timeDifferenceMinutes.toInt() * 0.45 // per minute points for running
//                            val storedPoints = mPrefs.getLong(prefCurrentRunningPoints, 0L)
//                            val currentPoints = storedPoints ?: 0L
//                            mPrefs.putLong(
//                                prefCurrentRunningPoints,
//                                (currentPoints + points).toLong()
//                            )
                            Log.d("ActivityRecognitionIntentService", "RUNNING points -> $points")
                            val records = dbHelper.getAllActivityPoints()
                            Log.d("ActivityRecognitionIntentService", "RUNNING records -> ${records.size}")
                            Log.d("ActivityRecognitionIntentService", "RUNNING first record -> ${records.first()}")
                            if (records.isEmpty()) {
                                dbHelper.addActivityPoints(0, 0, points.toInt(), 0, 0)
                            } else {
                                dbHelper.editActivityPoints(records.first().id,
                                    records.first().vehiclePoints ,
                                    records.first().bicyclePoints,
                                    records.first().runningPoints+points.toInt(),
                                    records.first().walkingPoints,
                                    records.first().stillPoints,
                                )
                            }
                        }

                        "WALKING" -> {
                            val points =
                                timeDifferenceMinutes.toInt() * 0.3 // per minute points for walking
//                            val storedPoints = mPrefs.getLong(prefCurrentWalkingPoints, 0L)
//                            val currentPoints = storedPoints ?: 0L
//                            mPrefs.putLong(
//                                prefCurrentWalkingPoints,
//                                (currentPoints + points).toLong()
//                            )
                            Log.d("ActivityRecognitionIntentService", "WALKING points -> $points")
                            val records = dbHelper.getAllActivityPoints()
                            Log.d("ActivityRecognitionIntentService", "WALKING records -> ${records.size}")
                            Log.d("ActivityRecognitionIntentService", "WALKING first record -> ${records.first()}")
                            if (records.isEmpty()) {
                                dbHelper.addActivityPoints(0, 0, 0, points.toInt(), 0)
                            } else {
                                dbHelper.editActivityPoints(records.first().id,
                                    records.first().vehiclePoints ,
                                    records.first().bicyclePoints,
                                    records.first().runningPoints,
                                    records.first().walkingPoints+points.toInt(),
                                    records.first().stillPoints,
                                )
                            }
                        }

                        "STILL" -> {
                            val points =
                                timeDifferenceMinutes.toInt() * 0.1 // per minute points for still
//                            val storedPoints = mPrefs.getLong(prefCurrentStillPoints, 0L)
//                            val currentPoints = storedPoints ?: 0L
//                            mPrefs.putLong(
//                                prefCurrentStillPoints,
//                                (currentPoints + points).toLong()
//                            )
                            Log.d("ActivityRecognitionIntentService", "STILL points -> $points")
                            val records = dbHelper.getAllActivityPoints()
                            Log.d("ActivityRecognitionIntentService", "STILL records -> ${records.size}")
                            Log.d("ActivityRecognitionIntentService", "STILL first record -> ${records.first()}")
                            if (records.isEmpty()) {
                                dbHelper.addActivityPoints(0, 0, 0,  0,points.toInt())
                            } else {
                                dbHelper.editActivityPoints(records.first().id,
                                    records.first().vehiclePoints ,
                                    records.first().bicyclePoints,
                                    records.first().runningPoints,
                                    records.first().walkingPoints,
                                    records.first().stillPoints+points.toInt(),
                                )
                            }
                        }

                        else -> println("UNKNOWN activity")
                    }
                }
            }
            dbHelper.addActivity(
                activityData.type,
                activityData.confidence,
                System.currentTimeMillis().toString()
            )
            //dbHelper.getRecords()
        } else {
            Log.d("ActivityRecognitionIntentService", "activity fetched ${activityData.type}")
        }


        var prefsKey: String
        var prefsValue: String
        try {
            prefsKey = Constants.ACTIVITY_DATA_PREFS_KEY
            prefsValue = jsonConverter.toJson(activityData)
        } catch (e: Exception) {
            prefsKey = Constants.ACTIVITY_ERROR_PREFS_KEY
            prefsValue = ErrorCodes.ACTIVITY_DATA_ENCODING_FAILED.toString()
        }

        val prefs = getSharedPreferences(
            Constants.ACTIVITY_RECOGNITION_RESULT_PREFS_NAME, Context.MODE_PRIVATE
        ) ?: return
        with(prefs.edit()) {
            putString(prefsKey, prefsValue)
            commit()
        }
    }
}
